#!/bin/bash

set -e

###################################### Setting

# 要 backup 的地方
SNAP_MOUNTPOINT="/"

# 自定 label (/mnt/urbackup_snaps/$SNAP_ID)
SNAP_ID="rootfs"

###################################### Code


#### 0 ####

# mount snapshoot 的位置
SNAP_DEST="/mnt/urbackup_snaps/$SNAP_ID"

if [ -e "$SNAP_DEST" ]
then
	echo "mount point $SNAP_DEST exist"
	exit 1
fi

mkdir -p $SNAP_DEST

#### 1 ####

DEVICE=$(df -T -P | egrep " ${SNAP_MOUNTPOINT}\$" | head -n 1 | tr -s " " | cut -d" " -f1)
FSTYPE=$(df -T -P | egrep " ${SNAP_MOUNTPOINT}\$" | head -n 1 | tr -s " " | cut -d" " -f2)

echo "DEVICE = $DEVICE"
echo "FSTYPE = $FSTYPE"

#### 2 ####

if [ "x$TYPE" = "xbtrfs" ]
then
    echo "Do not support btrfs"
    exit 1
fi

#### 3 ####

if ! modprobe dattobd
then
        echo "Dattobd kernel module not available"
        exit 1
fi

if ! modprobe loop
then
        echo "loop kernel module not available"
        exit 1
fi

#### 4 ####

NUM=0

while [ -e "/dev/datto$NUM" ]
do
        NUM=`expr $NUM + 1`
done

echo "Using /dev/datto$NUM ..."

sync

echo "Create snapshot ($SNAP_MOUNTPOINT/.datto_$SNAP_ID)"
dbdctl setup-snapshot "$DEVICE" "$SNAP_MOUNTPOINT/.datto_$SNAP_ID" $NUM

# For remove script
echo $NUM > ${SNAP_DEST}-num

#### 5 ####
# 加 storage for write
# mount: cannot mount /dev/datto0 read-only
# [937287.331534] XFS (datto0): recovery required on read-only device.
# [937287.331726] XFS (datto0): write access unavailable, cannot proceed.

# size 應該與 $DEVICE 一樣
DEV_SIZE=`blockdev --getsize /dev/datto$NUM`

echo "create overlay file ($SNAP_MOUNTPOINT/.overlay_$SNAP_ID)"
truncate -s100M $SNAP_MOUNTPOINT/.overlay_$SNAP_ID

LODEV=`losetup -f`
echo "loop = $LODEV"
if [ "x$LODEV" = x ]
then
	rm -f "${SNAP_DEST}-num"
	rm -f $SNAP_MOUNTPOINT/.overlay_$SNAP_ID
	dbdctl destroy $NUM
	exit 1
fi

losetup $LODEV $SNAP_MOUNTPOINT/.overlay_$SNAP_ID

echo "0 $DEV_SIZE snapshot /dev/datto$NUM $LODEV N 8" | dmsetup create "wsnap-$SNAP_ID"

echo "Mounting /dev/mapper/wsnap-$SNAP_ID..."

#### 6 ####

MOUNTOPTS="ro"

if [ $FSTYPE = "xfs" ]
then
    MOUNTOPTS="ro,nouuid"
fi

#### 8 ####

if ! mount -o $MOUNTOPTS /dev/mapper/wsnap-$SNAP_ID $SNAP_DEST
then
    echo "Mounting filesystem failed"
    rm "${SNAP_DEST}-num"
    rmdir "$SNAP_DEST"
    dmsetup remove "wsnap-$SNAP_ID"
    losetup -d $LODEV
    rm $SNAP_MOUNTPOINT/.overlay_$SNAP_ID
    dbdctl destroy $NUM
    exit 1
fi

echo "SNAPSHOT = $SNAP_DEST"

